@@ -3,6 +3,8 @@ package Lingua::Jspell;
 use warnings;
 use strict;
 
+use 5.008001;
+
 use POSIX qw(locale_h);
 setlocale(LC_CTYPE, "pt_PT");
 use locale;
@@ -15,13 +17,16 @@ our %EXPORT_TAGS = (basic => [qw.onethat verif ok any2str hash2str.],
 use File::Which qw/which/;
 use IPC::Open3;
 
+
 =head1 NAME
 
+=encoding utf8
+
 Lingua::Jspell - Perl interface to the Jspell morphological analyser.
 
 =cut
 
-our $VERSION = '1.50';
+our $VERSION = '1.51';
 our $JSPELL;
 our $JSPELLLIB;
 our $MODE = { nm => "af", flags => 0 };
@@ -167,9 +172,9 @@ sub fea{
       for(split(/[,;] /,$clas)){
         ($rad,$cla)= m{(.+?)\!:*(.*)$};
 
-	# No sei porqu, mas acontece por vezes de $cla ser 'undef'
-	# No sei bem o que devemos fazer... de momento, estou simplesmente
-	# a passar o cdigo  frente.
+	# Não sei porquê, mas acontece por vezes de $cla ser 'undef'
+	# Não sei bem o que devemos fazer... de momento, estou simplesmente
+	# a passar o código à frente.
 	if ($cla) {
 	  if ($cla =~ s/\/(.*)$//) { $flags = $1 }
 	  else                     { $flags = "" }
@@ -206,6 +211,11 @@ sub fea{
 
 =head2 flags
 
+returns the set of morphological flag associated with the word.
+Each flag is related with a set of morphological rules.
+
+ @f = flags("gato")
+
 =cut
 
 sub flags {
@@ -298,6 +308,8 @@ verifies the Feature Structure Pattern used.
 
    $analysis = onethat( { CAT=>'adj' }, @features);
 
+   $analysis = onethat( { CAT=>'adj' }, $pt->fea("espanhol"));
+
 =cut
 
 sub onethat {
@@ -375,12 +387,12 @@ Note: This function is specific for the Portuguese jspell dictionary
 
 =cut
 
-# NOTA: Esta funcao  especfica da lngua TUGA!
+# NOTA: Esta funcao é específica da língua TUGA!
 sub _cat2small {
   my %b = @_;
 
   if ($b{'CAT'} eq 'art') {
-    # Artigos: o lxico j prev todos...
+    # Artigos: o léxico já prevê todos...
     # por isso, NUNCA SE DEVE CHEGAR AQUI!!!
     return "ART";
     # 16 tags
@@ -388,7 +400,7 @@ sub _cat2small {
   } elsif ($b{'CAT'} eq 'card') {
     # Numerais cardinais:
     return "DNCNP";
-    # o lxico j prev os que flectem (1 e 2); o resto  tudo neutro plural.
+    # o léxico já prevê os que flectem (1 e 2); o resto é tudo neutro plural.
 
   } elsif ($b{'CAT'} eq 'nord') {
     # Numerais ordinais:
@@ -431,7 +443,7 @@ sub _cat2small {
     return "\U$b{'CAT'}$b{'G'}$b{'N'}";
 
   } elsif ($b{'CAT'} eq 'np') {
-    # Nomes prprios:
+    # Nomes próprios:
     $b{'G'} = 'N' if $b{'G'} eq '_' || $b{'G'} eq '';
     $b{'N'} = 'N' if $b{'N'} eq '_' || $b{'N'} eq '';
     return "\U$b{'CAT'}$b{'G'}$b{'N'}";
@@ -465,11 +477,11 @@ sub _cat2small {
       $b{'T'} = 'N';
 
     } elsif ($b{'T'} eq 'ppa') {
-      # Particpio Passado
+      # Particípio Passado
       $b{'T'} = 'PP';
 
     } elsif ($b{'T'} eq 'g') {
-      # Gerndio
+      # Gerúndio
       $b{'T'} = 'G';
 
     } elsif ($b{'T'} eq 'p') {
@@ -477,15 +489,15 @@ sub _cat2small {
       $b{'T'} = 'IH';
 
     } elsif ($b{'T'} eq 'pp') {
-      # modo indicativo: pretrito Perfeito
+      # modo indicativo: pretérito Perfeito
       $b{'T'} = 'IP';
 
     } elsif ($b{'T'} eq 'pi') {
-      # modo indicativo: pretrito Imperfeito
+      # modo indicativo: pretérito Imperfeito
       $b{'T'} = 'II';
 
     } elsif ($b{'T'} eq 'pmp') {
-      # modo indicativo: pretrito Mais-que-perfeito
+      # modo indicativo: pretérito Mais-que-perfeito
       $b{'T'} = 'IM';
 
     } elsif ($b{'T'} eq 'f') {
@@ -497,7 +509,7 @@ sub _cat2small {
       $b{'T'} = 'SH';
 
     } elsif ($b{'T'} eq 'pic') {
-      # modo conjuntivo (Se): pretrito Imperfeito
+      # modo conjuntivo (Se): pretérito Imperfeito
       $b{'T'} = 'SI';
 
     } elsif ($b{'T'} eq 'fc') {
@@ -516,47 +528,51 @@ sub _cat2small {
       # modo Infinitivo (Pessoal ou Presente): 
       $b{'T'} = 'PI';
 
-      # Futuro conjuntivo? S se tiver um "se" antes! -> regras sintcticas...
-      # modo&tempo no previstos ainda...
+      # Futuro conjuntivo? Só se tiver um "se" antes! -> regras sintácticas...
+      # modo&tempo não previstos ainda...
 
     } else {
       $b{'T'} = '_UNKNOWN';
     }
 
-    # converter 'P=1_3' em 'P=_': provisrio(?)!
-    $b{'P'} = '_' if $b{'P'} eq '1_3'; # nico stio com '_' como rhs!!!
+    # converter 'P=1_3' em 'P=_': provisório(?)!
+    $b{'P'} = '_' if $b{'P'} eq '1_3'; # único sítio com '_' como rhs!!!
 
-    return "\U$b{'CAT'}$b{'T'}$b{'G'}$b{'P'}$b{'N'}";
-    #                               Gnero, s para VPP.
+     
+    if($b{T} eq "vpp"){ return "\U$b{'CAT'}$b{'T'}$b{'G'}$b{'P'}$b{'N'}";}
+    else { return "\U$b{'CAT'}$b{'T'}$b{'P'}$b{'N'}";}
+
+
+    #                               Género, só para VPP.
     # +/- 70 tags
 
   } elsif ($b{'CAT'} eq 'prep') {
-    # Preposies:
+    # Preposições¹:
     return "\UP";
 
   } elsif ($b{'CAT'} eq 'adv') {
-    # Advrbios:
+    # Advérbios²:
     return "\UADV";
 
   } elsif ($b{'CAT'} eq 'con') {
-    # Conjunes:
+    # Conjunções²:
     return "\UC";
 
   } elsif ($b{'CAT'} eq 'in') {
-    # Interjeies:
+    # Interjeições¹:
     return "\UI";
 
-    # : no sei se a tag devia ser to atmica, mas para j no h confuso!
+    # ¹: não sei se a tag devia ser tão atómica, mas para já não há confusão!
 
   } elsif ($b{'CAT'} =~ m/^cp(.*)/) {
-    # Contraces:
+    # Contracções¹:
     $b{'G'} = 'N' if $b{'G'} eq '_';
     $b{'N'} = 'N' if $b{'N'} eq '_';
     return "\U&$b{'G'}$b{'N'}";
 
-    # : falta estruturar estes no prprio dicionrio...
-    # Palavras do dicionrio com categoria vazia ou sem categoria,
-    # palavras no existentes ou sequncias aleatrias de caracteres:
+    # ²: falta estruturar estes no próprio dicionário...
+    # Palavras do dicionário com categoria vazia ou sem categoria,
+    # palavras não existentes ou sequências aleatórias de caracteres:
 
   } elsif ($b{'CAT'} eq '') {
     return "\UUNDEFINED";
@@ -568,6 +584,12 @@ sub _cat2small {
 
 =head2 featags
 
+Given a word, returns a set of analysis. Each analysis is a morphosintatic tag
+
+ @l= $pt->featags("lindas") 
+   JFS , ...
+
+
 =cut
 
 sub featags{
@@ -575,11 +597,30 @@ sub featags{
   return (map {_cat2small(%$_)} ($self->fea($palavra)));
 }
 
+=head2 featagsrad
+
+Given a word, returns a set of analysis. Each analysis is a morphosintatic tag
+and the lemma information
+
+ @l= $pt->featagsrad("lindas") 
+   JFS:lindo , ...
+
+=cut
+
+sub featagsrad{
+  my ($self, $palavra) = @_;
+
+  return (map {_cat2small(%$_).":$_->{rad}"} ($self->fea($palavra)));
+}
+
+
 =head2 ok
 
  # ok: cond:fs x ele:fs-set -> bool
  # exist x in ele : verif(cond , x)
 
+ if(ok({CAT=>"adj"},$pt->fea("linda"))) { ... }
+
 =cut
 
 sub ok {
@@ -612,6 +653,10 @@ sub mkradtxt {
 
 =head2 any2str
 
+ Lingua::Jspell::any2str($ref)
+ Lingua::Jspell::any2str($ref,$indentation)
+ Lingua::Jspell::any2str($ref,"compact")
+
 =cut
 
 sub any2str {
@@ -625,6 +670,14 @@ sub any2str {
     } else {
       return "$r"
     }
+  } elsif ($i eq "f1") {
+    if (ref($r) eq "HASH") {
+      return "{". hash2str($r,"f1") . "}"
+    } elsif (ref($r) eq "ARRAY") {
+      return "[ " . join("  ,\n  ", map (any2str($_,"compact"), @$r)) . "]" 
+    } else {
+      return "$r"
+    }
   } else {
     my $ind = ($i >= 0)? (" " x $i) : "";
     if (ref($r) eq "HASH") {
@@ -649,6 +702,11 @@ sub hash2str {
       $c .= any2str($_,$i). "=". any2str($r->{$_},$i). ",";
     }
     chop($c);
+  } elsif ($i eq "f1") {
+    for (keys %$r) {
+      $c .= "\n  ", any2str($_,"compact"). "=". any2str($r->{$_},"compact"). "\n";
+    }
+    chop($c);
   } else {
     for (keys %$r) {
       $c .= "\n". any2str($_,$i). " => ". any2str($r->{$_},-$i);
@@ -660,7 +718,7 @@ sub hash2str {
 =head1 AUTHOR
 
 Jose Joao Almeida, C<< <jj@di.uminho.pt> >>
-Alberto Simes, C<< <ambs@di.uminho.pt> >>
+Alberto Simões, C<< <ambs@di.uminho.pt> >>
 
 =head1 BUGS
 
